<?php

namespace Drupal\datafield\Plugin\DataField\FieldWidget;

use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Plugin implementation of the 'number format' widget.
 */
#[FieldWidget(
  id: 'number_format',
  label: new TranslatableMarkup('Number format'),
  field_types: ['integer', 'numeric', 'float'],
)]
class NumberFormatWidget extends NumberWidget {

  /**
   * {@inheritdoc}
   */
  public function getFormElement(&$element, $item = NULL, $setting = []) {
    parent::getFormElement($element, $item, $setting);
    if ($setting['type'] == 'integer') {
      $limit = [
        // Signed.
        [
          'big' => ['min' => -9223372036854775808, 'max' => 9223372036854775807],
          'normal' => ['min' => -2147483648, 'max' => 2147483647],
          'medium' => ['min' => -8388608, 'max' => 8388607],
          'small' => ['min' => -32768, 'max' => 32767],
          'tiny' => ['min' => -128, 'max' => 127],
        ],
        // Unsigned.
        [
          'big' => ['min' => 0, 'max' => 18446744073709551615],
          'normal' => ['min' => 0, 'max' => 4294967295],
          'medium' => ['min' => 0, 'max' => 16777215],
          'small' => ['min' => 0, 'max' => 65535],
          'tiny' => ['min' => 0, 'max' => 255],
        ],
      ];
      $unsigned = $setting['unsigned'] ?? 0;
      $size = $setting['size'] ?? 0;
      if (isset($limit[$unsigned][$size])) {
        $element['#attributes']['min'] = $limit[$unsigned][$size]['min'];
        $element['#attributes']['max'] = $limit[$unsigned][$size]['max'];
      }
      $element['#attributes']['class'][] = 'text-end text-align-right';
    }
    if (in_array($setting['type'], ['decimal', 'float', 'numeric'])) {
      $element['#maxlength'] = $setting["precision"] + $setting["scale"] + 1;
    }
    $element['#type'] = 'textfield';
    $element['#attributes']['class'][] = 'text-format-number';
    $element['#attributes']['onfocus'] = 'use_number(this)';
    $element['#attributes']['onblur'] = 'use_text(this)';
    $element['#attributes']['pattern'] = '^-?[0-9.,-]*$';
    $element['#attached']['library'][] = 'datafield/formatnumber';
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues($value, array $form, FormStateInterface $form_state) {
    return preg_replace('/[^\d.-]/', '', $value);
  }

}
